import os
import time

import requests

API_VERSION = os.environ.get("DEFAULT_GRAPH_API_VERSION") if os.environ.get("DEFAULT_GRAPH_API_VERSION") else "v9.0"
BASE_URL = "https://graph.facebook.com/%s/%s"


def retry_strategy_print(r, url, ts):
    print(url)
    print(ValueError("Error when requesting graph api: %s" % r.text))
    print("Retry in %s seconds..." % ts)
    time.sleep(ts)


def post(system_user, endpoint, params, time_sleep=2):
    ts = time_sleep
    url = BASE_URL % (API_VERSION, endpoint)
    params["access_token"] = system_user.access_token
    r = requests.post(url, params=params)
    if r.status_code != 200:
        if ts <= 256:
            retry_strategy_print(r, url, ts)
            post(system_user, endpoint, params, time_sleep=2 * ts)
        else:
            raise ValueError("Error when requesting graph api: %s \n url: %s" % (r.text, url))
    result = r.json()
    return result.get('report_run_id')


def get_report_status(system_user, endpoint, params, time_sleep=2):
    ts = time_sleep
    url = BASE_URL % (API_VERSION, endpoint)
    params["access_token"] = system_user.access_token
    r = requests.get(url, params=params)
    if r.status_code != 200:
        if ts <= 256:
            retry_strategy_print(r, url, ts)
            return get_report_status(system_user, endpoint, params, time_sleep=2 * ts)
        else:
            raise ValueError("Error when requesting graph api: %s \n url: %s" % (r.text, url))
    return r.json()


def get(system_user, endpoint, params, time_sleep=2):
    ts = time_sleep
    url = BASE_URL % (API_VERSION, endpoint)
    params["access_token"] = system_user.access_token
    data = []
    r = requests.get(url, params=params)
    if r.status_code != 200:
        if ts <= 256:
            retry_strategy_print(r, url, ts)
            get(system_user, endpoint, params, time_sleep=2 * ts)
        else:
            raise ValueError("Error when requesting graph api: %s \n url: %s" % (r.text, url))
    result = r.json()
    if not result.get("data"):
        return []
    data = data + result.get("data")
    paging = True
    c = 0
    while paging:
        c = c + 1
        print("Paging: " + str(c))
        if result.get("paging"):
            if result["paging"].get("next"):
                r = requests.get(result["paging"]["next"])
                result = r.json()
                if result.get("data"):
                    data = data + result.get("data")
                else:
                    paging = False
            else:
                paging = False
        else:
            paging = False
    return data





def init(facebook, params):
    table_name = "%s.app_system_user" % facebook.config.get("schema_name")
    columns_name = ["id", "app_id_name", "app_secret_name", "access_token_name"]
    data = {
        "table_name": table_name,
        "columns_name": columns_name,
        "rows": [[params.get(c) for c in columns_name]]
    }
    facebook.transactional_dbstream.send_data(data,replace=False)



import datetime
import hashlib
import logging

import numpy as np

import pandas as pd

from pyfbook.SystemUser import SystemUser
from pyfbook.core.marketing.extract.api import get_report_status, get

time_increment_mapping = {
    "month": "month",
    "day": "1",
    "week": "week",
    "quarter": "quarter",
    "year": "year",
    "lifetime": "maximum",
    "maximum": "maximum"
}
SPECIAL_ACTIONS = [
    "video_10_sec_watched_actions",
    "video_30_sec_watched_actions",
    "video_avg_time_watched_actions",
    "video_p25_watched_actions",
    "video_p50_watched_actions",
    "video_p75_watched_actions",
    "video_p95_watched_actions",
    "video_p100_watched_actions",
    "video_play_actions",
    "video_thruplay_watched_actions",
    "cost_per_thruplay"
]


def treat_actions(row, r):
    actions = row.get('actions')
    if r.get('specific_actions'):
        specific_actions = r.get('specific_actions')
        if actions:
            for action in actions:
                if "action_" + action["action_type"].replace('.', '_') in specific_actions:
                    row["action_" + action["action_type"].replace('.', '_')] = action["value"]
            del row['actions']
    elif actions:
        for action in actions:
            row["action_" + action["action_type"].replace('.', '_')] = action["value"]
        del row['actions']
    return row


def treat_action_values(row):
    actions = row.get('action_values')
    if actions:
        for action in actions:
            row["action_" + action["action_type"].replace('.', '_') + '_value'] = action["value"]
        del row['action_values']
    return row


def treat_special_action(row, action_name):
    action = row.get(action_name)
    if action:
        row[action_name + '_' + action[0].get('action_type')] = action[0]["value"]
        del row[action_name]
    return row


def make_date(date_start, time_increment):
    if time_increment:
        return date_start
    print('Error time increment not specified in make_date function')
    exit()


def make_batch_id(date, account_id, campaign_id=None, adset_id=None, ad_id=None):
    chi = "_".join(k for k in [str(campaign_id), str(adset_id), str(ad_id)])
    return hashlib.sha224((date + str(account_id) + str(chi)).encode()).hexdigest()


def check_and_fetch_reports(facebook, reports_to_fetch=None):
    logging.info("Start fetching reports...")
    table_name = '%s.%s' % (
        facebook.schema_name if facebook.schema_name else facebook.config.get('schema_name'), 'report_async')
    created_at_min = str(datetime.datetime.now() + datetime.timedelta(hours=-3))[:19]
    all_status = ['Job Completed', 'Job Failed', 'Job Skipped', 'Job Deleted']
    if not reports_to_fetch:
        query = '''SELECT * FROM %s WHERE (status not in ('%s') or status is NULL or result_fetch is NULL) and created_at >= '%s' ''' % (
            table_name, "','".join(all_status), created_at_min)
    else:
        query = '''SELECT * FROM %s WHERE report_run_id in ('%s') ''' % (
            table_name, "','".join(reports_to_fetch))
    all_reports = facebook.transactional_dbstream.execute_query(query=query)
    job_not_completed_yet = False
    if not all_reports:
        return job_not_completed_yet
    for r in all_reports:
        if r["status"] != 'Job Completed':
            result = get_report_status(
                system_user=SystemUser.get(facebook=facebook, _id=r["app_system_user_id"]),
                endpoint=r['report_run_id'],
                params={}
            )
            status = result['async_status']
        else:
            status = r["status"]
        if status == 'Job Completed':
            data = _fetch_report(facebook=facebook, r=r)
            _send_data_fetch(
                data=data,
                time_increment=r["time_increment"],
                report_name=r["report_name"],
                facebook=facebook
            )
            r["result_fetch"] = str(datetime.datetime.now())[:19]
        else:
            job_not_completed_yet = True
            r["result_fetch"] = None
        r["status"] = status
        _update_report_status(r, facebook=facebook)
    return job_not_completed_yet


def _fetch_report(facebook, r):
    data = get(
        system_user=SystemUser.get(facebook=facebook, _id=r["app_system_user_id"]),
        endpoint=str(r['report_run_id']) + '/insights',
        params={}
    )
    result_data = []
    for row in data:
        row['date'] = make_date(row['date_start'], r["time_increment"])
        row['batch_id'] = make_batch_id(row['date'], account_id=row['account_id'], campaign_id=row.get("campaign_id"),
                                        adset_id=row.get("adset_id"), ad_id=row.get("ad_id"))
        row = treat_actions(row, r)
        row = treat_action_values(row)
        for e in SPECIAL_ACTIONS:
            row = treat_special_action(row, action_name=e)
        result_data.append(row)
    return result_data


def _send_data_fetch(data, time_increment, report_name, facebook):
    if not data:
        return 0
    table_name = '%s.%s_%s' % (
        facebook.schema_name if facebook.schema_name else facebook.config.get('schema_name'), report_name,
        time_increment)
    facebook.dbstream.send(data={'table_name': table_name, 'data': data}, replace=False)


def _clean_data_fetch(facebook, dict_data, table_name):
    batch_ids = list(set([d['batch_id'] for d in dict_data]))
    query = '''DELETE FROM %s WHERE batch_id in ('%s')''' % (table_name, "','".join(batch_ids))
    try:
        facebook.dbstream.execute_query(query=query)
    except Exception:
        pass


def _update_report_status(report, facebook):
    table_name = '%s.%s' % (
        facebook.schema_name if facebook.schema_name else facebook.config.get('schema_name'), 'report_async')
    updated_at = str(datetime.datetime.now())[:19]

    # noinspection SqlNoDataSourceInspection
    query = '''
              UPDATE %s
              SET updated_at = '%s' ,
              status = '%s',
              result_fetch = '%s'
              WHERE report_run_id = '%s'
    ''' % (
        table_name, updated_at, report["status"], report["result_fetch"], report["report_run_id"])
    facebook.transactional_dbstream.execute_query(query)



import datetime
import json

DATE_FORMAT = "%Y-%m-%d"


def date_to_string(date):
    return date.strftime(DATE_FORMAT)


def string_to_date(string):
    return datetime.datetime.strptime(string, DATE_FORMAT)


def segment_month_date(start, end):
    """
    start : YYYY-MM-DD
    end : YYYY-MM-DD
    """
    start_datetime = string_to_date(start)
    end_datetime = string_to_date(end)
    result = []

    start_datetime = start_datetime.replace(day=1)
    end_datetime = end_of_month(end_datetime)

    while start_datetime != end_datetime + datetime.timedelta(days=1) and start_datetime <= datetime.datetime.today():
        inter = [date_to_string(start_datetime),
                 date_to_string(end_of_month(start_datetime))]
        result.append(inter)
        start_datetime = add_a_month(start_datetime)
    return result


def segment_year_date(start, end):
    """
    start : YYYY-MM-DD
    end : YYYY-MM-DD
    """
    start_datetime = string_to_date(start)
    end_datetime = string_to_date(end)
    result = []

    start_datetime = start_datetime.replace(day=1, month=1)
    end_datetime = end_datetime.replace(day=1, month=1, year=end_datetime.year + 1) + datetime.timedelta(days=-1)

    while start_datetime != end_datetime + datetime.timedelta(days=1) and start_datetime <= datetime.datetime.today():
        inter = [date_to_string(start_datetime),
                 date_to_string(start_datetime.replace(year=start_datetime.year + 1) + datetime.timedelta(days=-1))]
        result.append(inter)
        start_datetime = start_datetime.replace(year=start_datetime.year + 1)
    return result


def segment_quarter_date(start, end):
    """
    start : YYYY-MM-DD
    end : YYYY-MM-DD
    """
    start_datetime = string_to_date(start)
    end_datetime = string_to_date(end)
    result = []

    start_datetime = start_of_quarter(start_datetime)
    end_datetime = end_of_quarter(end_datetime)

    while start_datetime != end_datetime + datetime.timedelta(days=1) and start_datetime <= datetime.datetime.today():
        inter = [date_to_string(start_datetime),
                 date_to_string(end_of_quarter(start_datetime))]
        result.append(inter)
        start_datetime = add_a_quarter(start_datetime)
    return result


def segment_week_date(start, end):
    """
    start : YYYY-MM-DD
    end : YYYY-MM-DD
    """
    start_datetime = string_to_date(start)
    end_datetime = string_to_date(end)
    result = []

    start_datetime = start_datetime - datetime.timedelta(days=start_datetime.weekday())
    end_datetime = end_datetime - datetime.timedelta(days=end_datetime.weekday()) + datetime.timedelta(days=6)

    while start_datetime != end_datetime + datetime.timedelta(days=1) and start_datetime <= datetime.datetime.today():
        inter = [date_to_string(start_datetime),
                 date_to_string(start_datetime + datetime.timedelta(days=6))]
        result.append(inter)
        start_datetime = start_datetime + datetime.timedelta(days=7)
    return result


def add_a_month(date):
    try:
        nextmonthdate = date.replace(month=date.month + 1, day=1)
    except ValueError:
        if date.month == 12:
            nextmonthdate = date.replace(year=date.year + 1, month=1, day=1)
        else:
            # next month is too short to have "same date"
            # pick your own heuristic, or re-raise the exception:
            raise
    return nextmonthdate


def add_a_quarter(date):
    for i in range(3):
        date = add_a_month(date)
    return date


def remove_a_month(date):
    try:
        previousmonthdate = date.replace(month=date.month - 1, day=1)
    except ValueError:
        if date.month == 1:
            previousmonthdate = date.replace(year=date.year - 1, month=12, day=1)
        else:
            # next month is too short to have "same date"
            # pick your own heuristic, or re-raise the exception:
            raise
    return previousmonthdate


def end_of_month(date):
    return add_a_month(date) + datetime.timedelta(days=-1)


def end_of_quarter(date):
    while date.month not in [12, 3, 6, 9]:
        date = add_a_month(date)
    return end_of_month(date)


def start_of_quarter(date):
    while date.month not in [1, 4, 7, 10]:
        date = remove_a_month(date)
    return date.replace(day=1)


def define_date(year, month):
    since = datetime.date(year, month, 1).strftime(DATE_FORMAT)
    until = datetime.date.today().strftime(DATE_FORMAT)
    return since, until


def define_date_year(year):
    since = datetime.date(year, 1, 1)
    until = datetime.date(year + 1, 1, 1) - datetime.timedelta(days=1)
    since = since.strftime(DATE_FORMAT)
    until = until.strftime(DATE_FORMAT)
    return since, until


def since_until_to_time_range(since, until):
    time_range = {
        "since": since,
        "until": until
    }
    time_range = json.dumps(time_range)
    return time_range


def since_until_to_time_ranges(since, until, time_increment):
    if time_increment == 'week':
        return [since_until_to_time_range(since, until) for since, until in segment_week_date(since, until)]
    if time_increment == 'month':
        return [since_until_to_time_range(since, until) for since, until in segment_month_date(since, until)]
    if time_increment == 'quarter':
        return [since_until_to_time_range(since, until) for since, until in segment_quarter_date(since, until)]
    if time_increment == 'year':
        return [since_until_to_time_range(since, until) for since, until in segment_year_date(since, until)]
    return 0


def set_last_year():
    last_year = datetime.date.today().year
    return define_date_year(last_year)


def set_default():
    return define_date(2018, 1)


def set_lifetime():
    return define_date(2017, 1)


def set_since_until(date_window):
    try:
        since = date_window["since"]
        until = date_window["until"]
    except TypeError or AttributeError:
        function_to_run = globals()["set_" + date_window]
        since, until = function_to_run()
    return since, until



import os
from shutil import copyfile

import yaml
from dbstream import DBStream

from pyfbook.SystemUser import SystemUser
from pyfbook.core.ad_accounts.ad_accounts import process_ad_accounts, get_ad_accounts
from pyfbook.core.marketing.extract.reports.launch import launch
from pyfbook.core.marketing.extract.reports.save import save_reports
from pyfbook.core.marketing.transform_and_load.fetch import check_and_fetch_reports, time_increment_mapping


def launch_reports(facebook, start, end):
    for report in facebook.config["reports"]:
        report_name = report['name']
        for time_increment in report["time_increments"]:
            print('Launch report %s %s' % (report_name, time_increment))
            async_reports = launch(
                facebook=facebook,
                report=report,
                time_increment=time_increment_mapping[time_increment],
                start=start,
                end=end
            )
            save_reports(
                data=async_reports,
                facebook=facebook,
                time_increment=time_increment_mapping[time_increment],
                report_name=report_name
            )


class Facebook:
    def __init__(self,
                 config_path=None,
                 dbstream: DBStream = None,
                 transactional_dbstream: DBStream = None,
                 launch_jobs=True,
                 schema_name=None):
        self.config_path = config_path
        if not config_path:
            print('You need a config file')
            create_config_field = input('Do you want to generate a default config field? [Y/n]')
            if create_config_field == 'Y':
                f = open("config.yaml", "w")
                sample_config_path = os.path.dirname(os.path.realpath(__file__)) + '/core/config/sample_config.yaml'
                copyfile(sample_config_path, 'config.yaml')
                f.close()
                print('Config file is created, update it and add its path to this Facebook object')
                exit()
            self.config_path = None
        self.config = yaml.load(open(self.config_path), Loader=yaml.FullLoader)
        self.dbstream = dbstream
        self.transactional_dbstream = transactional_dbstream
        if self.transactional_dbstream is None:
            self.transactional_dbstream = dbstream
        self.launch_jobs = launch_jobs
        self.schema_name = schema_name

    def get(self, report_name=None, time_increment=None, start=None, end=None, list_account_ids=None,
            specific_reports_batch=None):
        if report_name:
            report = list(filter(lambda c: c["name"] == report_name, self.config["reports"]))
            if not report:
                print("No report with this name")
                exit()
            self.config["reports"] = report
        if time_increment:
            for i in range(len(self.config["reports"])):
                self.config["reports"][i]["time_increments"] = [time_increment]
        if list_account_ids:
            for i in range(len(self.config["reports"])):
                self.config["reports"][i]["ad_accounts"] = list_account_ids
        if specific_reports_batch:
            for sr in specific_reports_batch:
                for i in range(len(self.config["reports"])):
                    self.config["reports"][i]["ad_accounts"] = [sr.get('id')]
                launch_reports(self, sr.get('start'), end=sr.get('end'))
        elif self.launch_jobs:
            launch_reports(self, start, end)
        bool_ = True
        while bool_:
            bool_ = check_and_fetch_reports(self)

    def get_all_ad_accounts(self, list_account_ids=None):
        table_name = "%s.ad_accounts" % (self.schema_name if self.schema_name else self.config.get("schema_name"))
        system_users = SystemUser.all(facebook=self)
        ad_accounts = []
        for system_user in system_users:
            ad_accounts = ad_accounts + get_ad_accounts(system_user)
        process_ad_accounts(ad_accounts, table_name, self, list_account_ids=list_account_ids)

    def get_all_ad_accounts_upgrade(self):
        table_name = "%s.ad_accounts" % (self.schema_name if self.schema_name else self.config.get("schema_name"))
        system_users = SystemUser.all(facebook=self)
        ad_accounts = []
        for system_user in system_users:
            ad_accounts = ad_accounts + get_ad_accounts(system_user)
        data = {
            'data': ad_accounts,
            'table_name': table_name
        }
        self.dbstream.send(data=data, replace=False)

    def set_active_status_ad_accounts(self, list_account_ids):
        table_name = "%s.ad_accounts_dataflow_status" % (
            self.schema_name if self.schema_name else self.config.get("schema_name"))
        data = [{'account_id': i, 'active': list_account_ids[i]} for i in list_account_ids]
        data = {
            'data': data,
            'table_name': table_name
        }
        self.dbstream.send(data=data, replace=False)

    def create_app_system_user(self, app_id_name, app_secret_name, access_token_name):
        SystemUser.create(self, app_id_name, app_secret_name, access_token_name)
        print("App system user's configuration name created in database")



import datetime


def _clean_data(facebook, data, table_name):
    report_run_ids = list(set([d['report_run_id'] for d in data]))
    query = '''DELETE FROM %s WHERE report_run_id in ('%s')''' % (table_name, "','".join(report_run_ids))
    try:
        facebook.transactional_dbstream.execute_query(query=query)
    except Exception:
        pass


def save_reports(facebook, data, time_increment, report_name):
    if not data:
        return 0
    table_name = '%s.%s' % (facebook.schema_name if facebook.schema_name else facebook.config.get('schema_name'), 'report_async')
    reports = []
    created_at = str(datetime.datetime.now())[:19]
    for r in data:
        r['time_increment'] = time_increment
        r['report_name'] = report_name
        r['status'] = None
        r['result_fetch'] = None
        r['created_at'] = created_at
        r['updated_at'] = created_at
        reports.append(r)
    columns_name = [c for c in reports[0].keys()]
    _clean_data(
        facebook=facebook,
        data=reports,
        table_name=table_name
    )
    facebook.transactional_dbstream.send_data({
        "table_name": table_name,
        "columns_name": columns_name,
        "rows": [[r[c] for c in columns_name] for r in reports]
    }, replace=False)



import datetime
import os

from pyfbook.SystemUser import SystemUser
from pyfbook.core.marketing.extract.api import post
from pyfbook.core.marketing.extract.params import prepare_report_request
from pyfbook.core.marketing.extract.start_update_date import define_start_date, define_updated_time_filter
from pyfbook.core.marketing.extract.filters import add_active_filter_to_report, add_updated_time_filter_to_report
from pyfbook.core.marketing.extract.date import since_until_to_time_ranges, since_until_to_time_range


def _launch_insights(facebook, account, params, since, until, time_increment):
    endpoint = str(account['id']) + "/insights"
    if time_increment in ["week", "month", "quarter", "year"]:
        time_ranges = str(since_until_to_time_ranges(since, until, time_increment))
        params["time_ranges"] = time_ranges
    elif time_increment == "lifetime" or time_increment == 'maximum':
        params["date_preset"] = "maximum"
    else:
        time_range = since_until_to_time_range(since, until)
        params["time_range"] = time_range
        params["time_increment"] = time_increment
    try:
        data = post(system_user=SystemUser.get(facebook=facebook, _id=account["app_system_user_id"]), endpoint=endpoint,
                    params=params)
    except ValueError:
        return None
    return {
        "report_run_id": data,
        "app_system_user_id": account["app_system_user_id"],
        "account_id": account["id"],
        "start_report": since,
        "end_report": until
    }


def launch(facebook, report, time_increment, start, end):
    r = prepare_report_request(facebook, report)
    accounts = r["accounts"]
    del r["accounts"]
    data = []
    for account in accounts:
        if start is None:
            start_account = define_start_date(facebook, report, time_increment, account)
            if start_account is None:
                if os.environ.get('DEFAULT_START_DATE'):
                    start_account = str(datetime.datetime.strptime(os.environ.get('DEFAULT_START_DATE'), '%Y-%m-%d'))[
                                    :10]
                else:
                    start_account = str(
                        datetime.datetime.strptime(str(datetime.datetime.now())[:10], '%Y-%m-%d') - datetime.timedelta(
                            days=365))[:10]
        else:
            start_account = start
        if end is None:
            end_account = str(datetime.datetime.now())[:10]
        else:
            end_account = end

        if time_increment == 'lifetime':
            updated_time_filter = define_updated_time_filter(facebook, report, time_increment, account)
            if updated_time_filter:
                # Define params to check active assets
                params_active_assets = add_active_filter_to_report(r)

                # Define params to check updated assets
                updated_time_filter = int(datetime.datetime.timestamp(updated_time_filter))
                params_updated_assets = add_updated_time_filter_to_report(r, updated_time_filter)

                for p in [params_active_assets, params_updated_assets]:
                    data.append(_launch_insights(
                        facebook=facebook,
                        account=account,
                        params=p,
                        since=start_account,
                        until=end_account,
                        time_increment=time_increment
                    ))
            else:
                data.append(_launch_insights(
                    facebook=facebook,
                    account=account,
                    params=r,
                    since=start_account,
                    until=end_account,
                    time_increment=time_increment
                ))
        else:
            data.append(_launch_insights(facebook, account, r, start_account, end_account, time_increment))
    return data



import datetime
import os


class SystemUser:
    def __init__(self, id, app_id_name, app_secret_name, access_token_name):
        self.id = id
        self.app_id_name = app_id_name
        self.app_id = os.environ[app_id_name]
        self.app_secret_name = app_secret_name
        self.app_secret = os.environ[app_secret_name]
        self.access_token_name = access_token_name
        self.access_token = os.environ[access_token_name]

    @staticmethod
    def all(facebook):
        schema_name = facebook.schema_name if facebook.schema_name else facebook.config.get('schema_name')
        r = facebook.transactional_dbstream.execute_query(
            'SELECT id, app_id_name, app_secret_name, access_token_name FROM %s.app_system_user' % schema_name)
        return [SystemUser(**result) for result in r]

    @staticmethod
    def get(_id, facebook):
        schema_name = facebook.schema_name if facebook.schema_name else facebook.config.get('schema_name')
        r = facebook.transactional_dbstream.execute_query(
            'SELECT id, app_id_name, app_secret_name, access_token_name FROM %s.app_system_user WHERE id=%s' % (
            schema_name, _id))
        if not r:
            print('SystemUser does not exist')
            exit(1)
        return SystemUser(**r[0])

    @staticmethod
    def create(facebook, app_id_name, app_secret_name, access_token_name):
        schema_name = facebook.schema_name if facebook.schema_name else facebook.config.get('schema_name')
        try:
            max_id = facebook.transactional_dbstream.execute_query('SELECT max(id) as max_id FROM %s.app_system_user' % schema_name)[
                0].get('max_id')
        except:
            max_id = 1
        data = {
            "table_name": '%s.app_system_user' % schema_name,
            "columns_name": ['id', 'app_id_name', 'app_secret_name', 'access_token_name', 'created_at', 'updated_at'],
            "rows": [[max_id + 1, app_id_name, app_secret_name, access_token_name, datetime.datetime.now(),
                      datetime.datetime.now()]]
        }
        facebook.transactional_dbstream.send_data(data, replace=False)



import json


def add_active_filter_to_report(r):
    result = r.copy()
    result["filtering"] = r["filtering"] if r.get("filtering") else []

    if result["level"] == "ad":
        new_filter = {'field': 'ad.effective_status', 'operator': 'IN', 'value': ['ACTIVE']}
    elif result["level"] == "adset":
        new_filter = {'field': 'adset.effective_status', 'operator': 'IN', 'value': ['ACTIVE']}
    elif result["level"] == "campaign":
        new_filter = {'field': 'campaign.effective_status', 'operator': 'IN', 'value': ['ACTIVE']}
    elif result["level"] == "account":
        new_filter = ""
    else:
        new_filter = ""
    result["filtering"].append(new_filter)
    result["filtering"] = json.dumps(result["filtering"])
    return result


def add_updated_time_filter_to_report(r, updated_time_filter):
    result = r.copy()
    result["filtering"] = r["filtering"] if r.get("filtering") else []
    if result["level"] == "ad":
        new_filter = {'field': 'ad.updated_time', 'operator': 'AFTER', 'value': updated_time_filter}
    elif result["level"] == "adset":
        new_filter = {'field': 'adset.updated_time', 'operator': 'AFTER', 'value': updated_time_filter}
    elif result["level"] == "campaign":
        new_filter = {'field': 'campaign.updated_time', 'operator': 'AFTER', 'value': updated_time_filter}
    elif result["level"] == "account":
        new_filter = ""
    else:
        new_filter = ""
    result["filtering"].append(new_filter)
    result["filtering"] = json.dumps(result["filtering"])
    return result



import datetime


# noinspection SqlNoDataSourceInspection

def define_start_date(facebook, report, time_increment, account):
    table_name = '%s.%s' % (facebook.schema_name if facebook.schema_name else facebook.config.get('schema_name'), 'report_async')
    query = """
            SELECT max(end_report) as start_date
            FROM %s 
            WHERE account_id='%s' and time_increment='%s' and report_name='%s' and status='Job Completed'
            and created_at>=end_report
            """ % (table_name, account["id"], time_increment, report.get('name'))

    try:
        start_date = facebook.transactional_dbstream.execute_query(query)[0]["start_date"]
    except Exception:
        return None
    if start_date:
        if time_increment == '1':
            return str(datetime.datetime.strptime(str(start_date)[:10], '%Y-%m-%d') - datetime.timedelta(days=1))[:10]
        return str(datetime.datetime.strptime(str(start_date)[:10], '%Y-%m-%d') - datetime.timedelta(days=28))[:10]
    else:
        return None


# noinspection SqlNoDataSourceInspection

def define_updated_time_filter(facebook, report, time_increment, account):
    table_name = '%s.%s_%s' % (facebook.schema_name if facebook.schema_name else facebook.config.get('schema_name'), report.get('name'), time_increment)
    query = """
                SELECT max(updated_time) as updated_time
                FROM %s 
                WHERE account_id='%s'
            """ % (table_name, account["account_id"])
    try:
        updated_time = facebook.dbstream.execute_query(query)[0]["updated_time"]
    except Exception:
        return None
    if updated_time:
        return datetime.datetime.strptime(str(updated_time)[:10], '%Y-%m-%d') - datetime.timedelta(days=28)
    else:
        return None



import json


def prepare_report_request(facebook, report):
    result = dict()
    result["level"] = report["level"]
    fields = report["fields"].copy()
    if report.get("filtering"):
        result["filtering"] = report.get("filtering")
        result["filtering"] = json.dumps(result["filtering"])
    if report.get("action_attribution_windows"):
        result["action_attribution_windows"] = report.get("action_attribution_windows")
    if report.get("action_report_time"):
        result["action_report_time"] = report.get("action_report_time")
    if 'account_id' not in fields:
        fields.append('account_id')
    if "purchase" in fields:
        fields[fields.index("purchase")] = "actions"
    elif "total_actions" in fields:
        fields[fields.index("total_actions")] = "actions"
    if "video_view_10_sec" in fields:
        fields[fields.index("video_view_10_sec")] = "video_10_sec_watched_actions"
    if "updated_time" not in fields:
        fields.append("updated_time")
    fields = ", ".join(fields)
    result["fields"] = fields
    if report.get("breakdowns"):
        breakdowns = [b for b in report["breakdowns"]]
        breakdowns = ", ".join(breakdowns)
    else:
        breakdowns = None
    result["breakdowns"] = breakdowns
    if report.get('ad_accounts'):
        accounts = report.get('ad_accounts')
        query = "SELECT DISTINCT id, app_system_user_id, account_id FROM %s.ad_accounts WHERE id in ('%s')"
        query = query % (facebook.schema_name if facebook.schema_name else facebook.config["schema_name"], "','".join(accounts))
    else:
        query = "SELECT DISTINCT id, app_system_user_id, account_id FROM %s.ad_accounts WHERE active = True"
        query = query % facebook.schema_name if facebook.schema_name else facebook.config["schema_name"]
    accounts = facebook.dbstream.execute_query(query=query)
    result["accounts"] = accounts
    return result



from setuptools import setup, find_packages

with open('README.rst') as f:
    readme = f.read()

setup(
    name='pyfbook',
    version='0.3.17',
    description='Easily collect data from Facebook APIs',
    long_description=readme,
    keywords='collect data facebook api',
    packages=find_packages(exclude=('tests', 'docs')),
    python_requires='>=3',
    install_requires=[
        "dbstream>=0.0.14",
        "PyYAML>=5.1"
    ],
)



import pandas as pd
import numpy as np
from pyfbook.core.marketing.extract.api import get


def _clean(all_ids, table_name, facebook):
    query = "DELETE FROM %s WHERE id in ('%s')" % (table_name, "','".join(all_ids))
    try:
        facebook.dbstream.execute_query(query=query)
    except Exception:
        pass


def process_ad_accounts(ad_accounts, table_name, facebook, list_account_ids=None):
    if not ad_accounts:
        return 0
    dict_ad_accounts = pd.json_normalize(ad_accounts, sep='__').replace({np.nan: None}).to_dict(
        orient='records')
    columns = []
    all_ids = []
    unique_ad_accounts = []
    for r in dict_ad_accounts:
        if r["id"] in all_ids:
            continue
        elif list_account_ids and r["id"] not in list_account_ids:
            continue
        all_ids.append(r["id"])
        r["active"] = True
        unique_ad_accounts.append(r)
        for k in r.keys():
            if k not in columns:
                columns.append(k)
    data = {
        "table_name": table_name,
        "columns_name": columns,
        "rows": [[i.get(c) for c in columns] for i in unique_ad_accounts]
    }

    _clean(all_ids, table_name, facebook=facebook)
    facebook.dbstream.send_data(data=data, replace=False)


def get_ad_accounts(system_user):
    params = {
        "fields": "name,account_id,currency,partner"
    }
    result = get(system_user, "me/adaccounts", params)
    for r in result:
        r["app_system_user_id"] = system_user.id
    return result
